//--------------------------------------------------------------------- 
#include "BaseApp.h"
#include "BFindWindow.h" 
#include <Be.h> 
//--------------------------------------------------------------------- 
BaseApp::BaseApp() 
    :BApplication(APPLICATION_SIGNATURE) 
{ 
    EditorList=new BList(); 
    open_filepanel= new BFilePanel(); 
    findword="";
    replaceword="";
    ignorecase=false;
} 
//--------------------------------------------------------------------- 
BaseApp::~BaseApp() 
{ 
    delete open_filepanel; 
        
    for(int i=EditorList->CountItems()-1;i>=0;i--) 
    { 
        BAppMainWindow *targetwnd= 
                        ((BAppMainWindow *)EditorList->ItemAt(i)); 
        targetwnd->Lock();
        targetwnd->Quit(); 
        EditorList->RemoveItem(targetwnd); 
    } 
    delete EditorList; 
} 
//--------------------------------------------------------------------- 
BAppMainWindow *BaseApp::AddEditor(void) 
{ 
    BAppMainWindow *mainwnd=new BAppMainWindow( 
        BRect(MAINWINDOW_POSITION_LEFT,MAINWINDOW_POSITION_TOP, 
              MAINWINDOW_POSITION_WIDTH+MAINWINDOW_POSITION_LEFT-1, 
              MAINWINDOW_POSITION_HEIGHT+MAINWINDOW_POSITION_TOP-1), 
        MAINWINDOW_TITLE); 
        
    EditorList->AddItem((void *)mainwnd); 
    mainwnd->Show(); 

    return mainwnd; 
} 
//--------------------------------------------------------------------- 
void BaseApp::MessageReceived(BMessage *msg) 
{ 
    switch(msg->what) 
    {        
    case MSG_OPEN: 
        open_filepanel->Show(); 
        break; 
    case MSG_QUIT: 
        for(int i=EditorList->CountItems()-1;i>=0;i--) 
            ((BAppMainWindow *)EditorList->ItemAt(i))-> 
                                    PostMessage(B_QUIT_REQUESTED);                            
        break; 
    case MSG_CLOSED: 
        { 
            BAppMainWindow *targetwnd; 
                        
            msg->FindPointer("wnd",(void **)&targetwnd); 
                        
            if(targetwnd!=NULL) 
            {
                targetwnd->Lock(); 
                targetwnd->Quit(); 
                EditorList->RemoveItem(targetwnd); 

                if(EditorList->CountItems()==0) 
                    PostMessage(B_QUIT_REQUESTED); 
            } 
        } 
        break; 
    case MSG_ADDEDITOR: 
        AddEditor(); 
        break;
    case MSG_ACTIVATE:
        { 
            BAppMainWindow *targetwnd; 
                        
            msg->FindPointer("wnd",(void **)&targetwnd); 
                        
            if(targetwnd!=NULL) 
            { 
                EditorList->RemoveItem(targetwnd);
                EditorList->AddItem(targetwnd);
            }
        }
        break; 
    case MSG_SHOWFIND:
        {
            BFindWindow *findwnd=new BFindWindow( 
                BRect(FINDWINDOW_POSITION_LEFT,FINDWINDOW_POSITION_TOP, 
                      FINDWINDOW_POSITION_WIDTH+FINDWINDOW_POSITION_LEFT-1, 
                      FINDWINDOW_POSITION_HEIGHT+FINDWINDOW_POSITION_TOP-1), 
                FINDWINDOW_TITLE,findword,replaceword,ignorecase);
        
            findwnd->Show(); 
        }        
        break;
    case MSG_FIND:
    case MSG_REPLACEALL:
        {
            BAppMainWindow *targetwnd=(BAppMainWindow *)EditorList->
                                    ItemAt(EditorList->CountItems()-1);

            msg->FindString("findword",&findword);
            msg->FindString("replaceword",&replaceword);
            msg->FindBool("ignorecase",&ignorecase);
            if(targetwnd!=NULL)
            {
                targetwnd->Lock();
                targetwnd->Activate();
                if(msg->what==MSG_FIND)
                    targetwnd->FindWord(findword,ignorecase);
                else
                if(msg->what==MSG_REPLACEALL)
                    targetwnd->ReplaceAll(findword,
                                          replaceword,ignorecase);
                targetwnd->Unlock();
            }
        }
        break;
    case MSG_FINDNEXT:
    case MSG_REPLACEFINDNEXT:
    case MSG_REPLACEINSELECTION:
        {
            BAppMainWindow *targetwnd=(BAppMainWindow *)EditorList->
                                    ItemAt(EditorList->CountItems()-1);
            if(findword!="" && targetwnd!=NULL)
            {
                targetwnd->Lock();
                targetwnd->Activate();
                if(msg->what==MSG_REPLACEINSELECTION)
                    targetwnd->ReplaceInSelection(
                                     findword,replaceword,ignorecase);
                else
                {
                    if(msg->what==MSG_REPLACEFINDNEXT)
                        targetwnd->ReplaceWord(replaceword);
                    targetwnd->FindWord(findword,ignorecase);
                }
                targetwnd->Unlock();
            }
        }
        break;
    default: 
        BApplication::MessageReceived(msg); 
    } 
} 
//--------------------------------------------------------------------- 
void BaseApp::RefsReceived(BMessage *message) 
{ 
    type_code ref_type; 
    int32 ref_count; 
    entry_ref ref; 
    BEntry entry; 
        
    message->GetInfo("refs", &ref_type,&ref_count); 
    for(int i=0;i<ref_count;i++) 
    { 
        message->FindRef("refs",i,&ref); 
        if(entry.SetTo(&ref)==B_OK) 
        { 
            if(entry.IsFile()) 
            { 
                BAppMainWindow *targetwnd; 

                targetwnd=AddEditor(); 

                targetwnd->Lock(); 
                targetwnd->OpenFile(&entry); 
                targetwnd->Unlock(); 
            } 
        } 
    } 
} 
//--------------------------------------------------------------------- 
void BaseApp::ReadyToRun() 
{ 
    if(EditorList->CountItems()==0) 
        AddEditor(); 
} 
//---------------------------------------------------------------------